/***************************************************************************
                          smplsnd.cpp  -  description
                             -------------------
    begin                : Wed Jun 26 2002
    copyright            : (C) 2002 by Nobuhiro Kuroiwa
    email                : nobu@iastate.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <iostream>

#include "smplsnd.h"
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"

ChunkBank chunkbank;

//---------------------------------------------------------------------------
//	Constructor of SimpleSound Class
//---------------------------------------------------------------------------
SimpleSound::SimpleSound(){
	audio_frequency	= AUDIO_FREQUENCY;
	audio_format	= AUDIO_FORMAT;
	audio_mode		= AUDIO_MODE;
	audio_buffersize= AUDIO_BUFFERSIZE;
	paudio_chunk	= NULL;
	angle			= AUDIO_DEFAULT_ANGLE;
	distance		= AUDIO_DEFAULT_DISTANCE;
	mychunk			= true;
}

//---------------------------------------------------------------------------
//	Destructor of SimpleSound Class
//---------------------------------------------------------------------------
SimpleSound::~SimpleSound(){
	if( paudio_chunk != NULL && mychunk)
	{
		Mix_FreeChunk( paudio_chunk);
	}
}

//---------------------------------------------------------------------------
//	SimpleSound::play() plays sounds of chunk which is stored as paudio_chunk
//---------------------------------------------------------------------------
bool SimpleSound::play()
{
	int channelno;

	channelno = Mix_PlayChannelTimed( -1, paudio_chunk, 0, -1);

	if( channelno == -1)
	{
		cout << filename + " : Unable to play channel : " + Mix_GetError() << endl;
		return false;
	}
	
	if(!Mix_SetPosition( channelno, angle, distance)) {
		cout << "Mix_SetPosition Failed : " << Mix_GetError() << endl;
		// no position effect, is it ok?
	}
	return true;
}

//---------------------------------------------------------------------------
//	SimpleSound::load() loads file whose name is *pnameoffile.
//	to avoid loading same file multiple times, The chunk once stored is
//	registered to chunkbank. So 1st we check it.
//---------------------------------------------------------------------------
bool SimpleSound::load( string *pnameoffile)
{
 	//-------------------------------------------------------------------------
	//	Try chunkbank first. If no chunk is registered, load it
	//-------------------------------------------------------------------------
	paudio_chunk = chunkbank.getChunk( pnameoffile);

 	//-------------------------------------------------------------------------
	//	Note that if paudio_chunk == NULL, mychunk = true. But the default value
	//	true.	So I don't write the code.
 	//-------------------------------------------------------------------------
	if( paudio_chunk == NULL)

	{
		paudio_chunk = Mix_LoadWAV( pnameoffile->c_str());

		if( !paudio_chunk)
		{
			cout << " : Unable to Open Audio File : \"" + *pnameoffile + "\" : "
			 + Mix_GetError() << endl;
			return false;
		}
		mychunk = true;
	 	chunkbank.assignChunk( paudio_chunk, pnameoffile);
	}
	else
	{
		mychunk = false;
	}
	filename = *pnameoffile;

 	//-------------------------------------------------------------------------
	//	It's important to call Mix_QuerySpec before calling getSimpleSoundLength
	//	Because sometimes expected value differs from obtained value.
 	//-------------------------------------------------------------------------
	Mix_QuerySpec(&audio_frequency, &audio_format, &audio_mode);

	return true;
}

//---------------------------------------------------------------------------
//	setPosition parameters. angle is in 0 to 360, distance is in 0 to 255.
//	The sound source location will be set to the position.
//---------------------------------------------------------------------------
void	SimpleSound::setPosition( Sint16 new_angle, Uint8 new_dist)
{
	angle		= new_angle;
	distance	= new_dist;
}

//---------------------------------------------------------------------------
//	Default Constructor of ChunkAndName Class
//---------------------------------------------------------------------------
ChunkAndName::ChunkAndName(){}

//---------------------------------------------------------------------------
//	Constructor of ChunkAndName Class
//---------------------------------------------------------------------------
ChunkAndName::ChunkAndName( Mix_Chunk *new_chunk, string *pstr)
{
	chunkname	= *pstr;
	chunk		= new_chunk;
}

//---------------------------------------------------------------------------
//	Default Constructor of ChunkBank Class
//---------------------------------------------------------------------------
ChunkBank::ChunkBank()
{}

//---------------------------------------------------------------------------
//	member function of ChunkBank Class who check the name of chunk and then
//	if it is already loaded, returns the chunk, not loaded, returns NULL.
//---------------------------------------------------------------------------
Mix_Chunk *ChunkBank::getChunk( string *pstr_name)
{
	it_chunkandname = lst_chunkandname.begin();
	while( it_chunkandname != lst_chunkandname.end())
	{
		if( pstr_name->compare( (*it_chunkandname).chunkname) == 0)
		{
			return (*it_chunkandname).chunk;
		}

		it_chunkandname++;
	}
	return NULL;
}

//---------------------------------------------------------------------------
//	number funciton of ChunkBank. assign a new chunk and name. The name and
//	chunk will be stored as ChunkAndName object.
//---------------------------------------------------------------------------
bool ChunkBank::assignChunk( Mix_Chunk *new_chunk, string *pname)
{
	if( getChunk( pname) != NULL)
		return false;

	lst_chunkandname.push_back( ChunkAndName( new_chunk, pname));
	return true;
}

